"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Jimp2 = require("jimp");
const HexToRGB = (hex) => {
    const r = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return r ? { r: parseInt(r[1], 16), g: parseInt(r[2], 16), b: parseInt(r[3], 16) } : null;
};
const ClampPalette = function (options) {
    if (!options.colors || !options.colors.length) {
        return options.errorcallback(new Error("Palette must be provided."));
    }
    const realColors = options.colors.map(c => HexToRGB(c));
    const callback = (err, image) => {
        const w = image.getWidth(), h = image.getHeight();
        for (let x = 0; x < w; x++) {
            for (let y = 0; y < h; y++) {
                const myColor = Jimp2.intToRGBA(image.getPixelColor(x, y));
                let lowestDistance = 99999, lowestIdx = -1;
                for (let i = 0; i < realColors.length; i++) {
                    const testColor = realColors[i];
                    if (testColor === null) {
                        return options.errorcallback(new Error("Invalid palette. All values must be hexadecimal colors."));
                    }
                    const dr = testColor.r - myColor.r;
                    const dg = testColor.g - myColor.g;
                    const db = testColor.b - myColor.b;
                    const myDist = dr * dr + dg * dg + db * db;
                    if (myDist < lowestDistance) {
                        lowestDistance = myDist;
                        lowestIdx = i;
                    }
                }
                const newColor = realColors[lowestIdx];
                if (newColor === null) {
                    return options.errorcallback(new Error("Invalid palette. All values must be hexadecimal colors."));
                }
                const newColorNum = Jimp2.rgbaToInt(newColor.r, newColor.g, newColor.b, 255);
                image.setPixelColor(newColorNum, x, y);
            }
        }
        image.getBuffer(options.mimeType || "image/png", (err, image) => {
            if (err) {
                options.errorcallback(err);
            }
            else {
                options.callback(image);
            }
        });
    };
    if (options.imagePath) {
        Jimp2.read(options.imagePath, callback);
    }
    else if (options.imageBuffer) {
        Jimp2.read(options.imageBuffer, callback);
    }
    else {
        return options.errorcallback(new Error("No image provided."));
    }
};
exports.default = ClampPalette;
